@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2002-2003 by Koos Vriezen <koos ! vriezen ? xs4all ! nl>
+ * Copyright (C) 2002-2003 by Koos Vriezen <koos.vriezen@gmail.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -12,13 +12,13 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #include <algorithm>
 #include <functional>
-
+#include <config.h>
 #include <qcheckbox.h>
 #include <qtextedit.h>
 #include <qpushbutton.h>
@@ -33,6 +33,7 @@
 
 #include <kurlrequester.h>
 #include <klineedit.h>
+#include <kstatusbar.h>
 
 #include <kconfig.h>
 #include <kapplication.h>
@@ -41,63 +42,117 @@
 #include <klocale.h>
 #include <kcombobox.h>
 #include <kmessagebox.h>
+#include <kglobalsettings.h>
 
+#include "kmplayersource.h"
 #include "kmplayerconfig.h"
 #include "kmplayerpartbase.h"
 #include "kmplayerprocess.h"
-#include "kmplayersource.h"
-#include "kmplayerview.h"
+#include "playlistview.h"
 #include "pref.h"
 
+using namespace KMPlayer;
+
 static OutputDriver _ads[] = {
-    { "", i18n ("Default from MPlayer Config File") },
+    { "alsa,oss,sdl,arts", i18n ("Auto") },
     { "oss", i18n ("Open Sound System") },
     { "sdl", i18n ("Simple DirectMedia Layer") },
     { "alsa", i18n ("Advanced Linux Sound Architecture") },
     { "arts", i18n ("Analog Real-Time Synthesizer") },
+    { "jack", i18n ("JACK Audio Connection Kit") },
+    { "openal", i18n ("OpenAL") },
     { "esd", i18n ("Enlightened Sound Daemon") },
     { "alsa5", i18n ("Advanced Linux Sound Architecture v0.5") },
     { "alsa9", i18n ("Advanced Linux Sound Architecture v0.9") },
+    { "", i18n ("Use back-end defaults") },
     { 0, QString::null }
 };
 
 static OutputDriver _vds [] = {
-    { "xv", i18n ("XVideo") },
+    { "xv,sdl,x11", i18n ("Auto") },
     { "x11", i18n ("X11Shm") },
     { "xvidix", i18n ("XVidix") },
+    { "xvmc,xv", i18n ("XvMC") },
+    { "sdl", i18n ("SDL") },
+    { "gl", i18n ("OpenGL") },
+    { "gl2", i18n ("OpenGL MT") },
+    { "xv", i18n ("XVideo") },
     { 0, QString::null }
 };
 
 static const int ADRIVER_ARTS_INDEX = 4;
 
 
-KDE_NO_CDTOR_EXPORT KMPlayerSettings::KMPlayerSettings (KMPlayer * player, KConfig * config)
-  : configdialog (0L), m_config (config), m_player (player) {
+KDE_NO_CDTOR_EXPORT Settings::Settings (PartBase * player, KConfig * config)
+  : pagelist (0L), configdialog (0L), m_config (config), m_player (player) {
     audiodrivers = _ads;
     videodrivers = _vds;
+    colors [ColorSetting::playlist_background].title = i18n ("Playlist background");
+    colors [ColorSetting::playlist_background].option = "PlaylistBackground";
+    colors [ColorSetting::playlist_background].color = KGlobalSettings::baseColor ();
+    colors [ColorSetting::playlist_foreground].title = i18n ("Playlist foreground");
+    colors [ColorSetting::playlist_foreground].option = "PlaylistForeground";
+    colors [ColorSetting::playlist_foreground].color = KGlobalSettings::textColor();
+    colors [ColorSetting::console_background].title =i18n("Console background");
+    colors [ColorSetting::playlist_active].title = i18n("Playlist active item");
+    colors [ColorSetting::playlist_active].option = "PlaylistActive";
+    colors [ColorSetting::playlist_active].color = KGlobalSettings::linkColor();
+    colors [ColorSetting::console_background].option = "ConsoleBackground";
+    colors [ColorSetting::console_background].color = QColor (0, 0, 0);
+    colors [ColorSetting::console_foreground].title = i18n ("Console foreground");
+    colors [ColorSetting::console_foreground].option = "ConsoleForeground";
+    colors [ColorSetting::console_foreground].color = QColor (0xB2, 0xB2, 0xB2);
+    colors [ColorSetting::video_background].title = i18n ("Video background");
+    colors [ColorSetting::video_background].option = "VideoBackground";
+    colors [ColorSetting::video_background].color = QColor (0, 0, 0);
+    colors [ColorSetting::area_background].title = i18n ("Viewing area background");
+    colors [ColorSetting::area_background].option = "ViewingAreaBackground";
+    colors [ColorSetting::area_background].color = QColor (0, 0, 0);
+    colors [ColorSetting::infowindow_background].title = i18n ("Info window background");
+    colors [ColorSetting::infowindow_background].option ="InfoWindowBackground";
+    colors [ColorSetting::infowindow_background].color = KGlobalSettings::baseColor ();
+    colors [ColorSetting::infowindow_foreground].title = i18n ("Info window foreground");
+    colors [ColorSetting::infowindow_foreground].option ="InfoWindowForeground";
+    colors [ColorSetting::infowindow_foreground].color = KGlobalSettings::textColor();
+    fonts [FontSetting::playlist].title = i18n ("Playlist");
+    fonts [FontSetting::playlist].option = "PlaylistFont";
+    fonts [FontSetting::playlist].font = KGlobalSettings::generalFont();
+    fonts [FontSetting::playlist].font.setItalic (true);
+    fonts [FontSetting::infowindow].title = i18n ("Info window");
+    fonts [FontSetting::infowindow].option = "InfoWindowFont";
+    fonts [FontSetting::infowindow].font = KGlobalSettings::generalFont();
 }
 
-KDE_NO_CDTOR_EXPORT KMPlayerSettings::~KMPlayerSettings () {
+KDE_NO_CDTOR_EXPORT Settings::~Settings () {
     // configdialog should be destroyed when the view is destroyed
     //delete configdialog;
 }
 
-const char * strMPlayerGroup = "MPlayer";
-static const char * strGeneralGroup = "General Options";
+KDE_EXPORT const char * strMPlayerGroup = "MPlayer";
+const char * strGeneralGroup = "General Options";
 static const char * strKeepSizeRatio = "Keep Size Ratio";
+static const char * strRememberSize = "Remember Size";
+static const char * strAutoResize = "Auto Resize";
+static const char * strDockSysTray = "Dock in System Tray";
+static const char * strNoIntro = "No Intro";
+static const char * strVolume = "Volume";
 static const char * strContrast = "Contrast";
 static const char * strBrightness = "Brightness";
 static const char * strHue = "Hue";
 static const char * strSaturation = "Saturation";
 static const char * strURLList = "URL List";
 static const char * strSubURLList = "URL Sub Title List";
+static const char * strPrefBitRate = "Prefered Bitrate";
+static const char * strMaxBitRate = "Maximum Bitrate";
 //static const char * strUseArts = "Use aRts";
 static const char * strVoDriver = "Video Driver";
 static const char * strAoDriver = "Audio Driver";
-static const char * strShowConsole = "Show Console Output";
 static const char * strLoop = "Loop";
 static const char * strFrameDrop = "Frame Drop";
+static const char * strAdjustVolume = "Auto Adjust Volume";
+static const char * strAdjustColors = "Auto Adjust Colors";
 static const char * strAddConfigButton = "Add Configure Button";
+static const char * strAddPlaylistButton = "Add Playlist Button";
 static const char * strAddRecordButton = "Add Record Button";
 static const char * strAddBroadcastButton = "Add Broadcast Button";
 static const char * strPostMPlayer090 = "Post MPlayer 0.90";
@@ -151,43 +206,105 @@ static const char * strAutoPlayAfterRecording = "Auto Play After Recording";
 static const char * strAutoPlayAfterTime = "Auto Play After Recording Time";
 static const char * strRecordingCopy = "Recording Is Copy";
 
+KDE_NO_EXPORT void Settings::applyColorSetting (bool only_changed_ones) {
+    View *view = static_cast <View *> (m_player->view ());
+    if (!view) return;
+    for (int i = 0; i < int (ColorSetting::last_target); i++)
+        if (!only_changed_ones || colors[i].color != colors[i].newcolor) {
+            colors[i].color = colors[i].newcolor;
+            switch (ColorSetting::Target (i)) {
+                case ColorSetting::playlist_background:
+                   view->playList()->setPaletteBackgroundColor(colors[i].color);
+                   break;
+                case ColorSetting::playlist_foreground:
+                   view->playList()->setPaletteForegroundColor(colors[i].color);
+                   break;
+                case ColorSetting::playlist_active:
+                   view->playList()->setActiveForegroundColor (colors[i].color);
+                   break;
+                case ColorSetting::console_background:
+                   view->console()->setPaper (QBrush (colors[i].color));
+                   break;
+                case ColorSetting::console_foreground:
+                   view->console()->setColor(colors[i].color);
+                   break;
+                case ColorSetting::video_background:
+                   view->viewer ()->setBackgroundColor (colors[i].color);
+                   break;
+                case ColorSetting::area_background:
+                   view->viewArea()->setPaletteBackgroundColor(colors[i].color);
+                   break;
+                case ColorSetting::infowindow_background:
+                   view->infoPanel ()->setPaper (QBrush (colors[i].color));
+                  break;
+                case ColorSetting::infowindow_foreground:
+                  view->infoPanel()->setPaletteForegroundColor(colors[i].color);
+                  view->infoPanel ()->setColor (colors[i].color);
+                  break;
+                default:
+                    ;
+            }
+        }
+    for (int i = 0; i < int (FontSetting::last_target); i++)
+        if (!only_changed_ones || fonts[i].font != fonts[i].newfont) {
+            fonts[i].font = fonts[i].newfont;
+            switch (FontSetting::Target (i)) {
+                case FontSetting::playlist:
+                   view->playList ()->setFont (fonts[i].font);
+                   break;
+                case FontSetting::infowindow:
+                   view->infoPanel ()->setFont (fonts[i].font);
+                   break;
+                default:
+                    ;
+            }
+        }
+}
 
-KDE_NO_EXPORT void KMPlayerSettings::readConfig () {
-    KMPlayerView *view = static_cast <KMPlayerView *> (m_player->view ());
+View * Settings::defaultView () {
+    return static_cast <View *> (m_player->view ());
+}
 
+KDE_NO_EXPORT void Settings::readConfig () {
     m_config->setGroup (strGeneralGroup);
+    no_intro = m_config->readBoolEntry (strNoIntro, false);
     urllist = m_config->readListEntry (strURLList, ';');
     sub_urllist = m_config->readListEntry (strSubURLList, ';');
+    prefbitrate = m_config->readNumEntry (strPrefBitRate, 512);
+    maxbitrate = m_config->readNumEntry (strMaxBitRate, 1024);
+    volume = m_config->readNumEntry (strVolume, 20);
     contrast = m_config->readNumEntry (strContrast, 0);
     brightness = m_config->readNumEntry (strBrightness, 0);
     hue = m_config->readNumEntry (strHue, 0);
     saturation = m_config->readNumEntry (strSaturation, 0);
+    const QMap <QString, Source*>::const_iterator e = m_player->sources ().end ();
+    QMap <QString, Source *>::const_iterator i = m_player->sources().begin ();
+    for (; i != e; ++i)
+        backends[i.data()->name ()] = m_config->readEntry (i.data()->name ());
+    for (int i = 0; i < int (ColorSetting::last_target); i++)
+        colors[i].newcolor = colors[i].color = m_config->readColorEntry (colors[i].option, &colors[i].color);
+    for (int i = 0; i < int (FontSetting::last_target); i++)
+        fonts[i].newfont = fonts[i].font = m_config->readFontEntry (fonts[i].option, &fonts[i].font);
 
     m_config->setGroup (strMPlayerGroup);
     sizeratio = m_config->readBoolEntry (strKeepSizeRatio, true);
-    view->setKeepSizeRatio (sizeratio);
-    showconsole = m_config->readBoolEntry (strShowConsole, false);
-    view->setShowConsoleOutput (showconsole);
+    remembersize = m_config->readBoolEntry (strRememberSize, true);
+    autoresize = m_config->readBoolEntry (strAutoResize, true);
+    docksystray = m_config->readBoolEntry (strDockSysTray, true);
     loop = m_config->readBoolEntry (strLoop, false);
     framedrop = m_config->readBoolEntry (strFrameDrop, true);
+    autoadjustvolume = m_config->readBoolEntry (strAdjustVolume, true);
+    autoadjustcolors = m_config->readBoolEntry (strAdjustColors, true);
     mplayerpost090 = m_config->readBoolEntry (strPostMPlayer090, true);
     showcnfbutton = m_config->readBoolEntry (strAddConfigButton, true);
-    if (showcnfbutton)
-        view->buttonBar ()->configButton ()->show ();
-    else
-        view->buttonBar ()->configButton ()->hide ();
     showrecordbutton = m_config->readBoolEntry (strAddRecordButton, true);
     showbroadcastbutton = m_config->readBoolEntry (strAddBroadcastButton, true);
-    if (showrecordbutton)
-        view->buttonBar ()->recordButton ()->show ();
-    else
-        view->buttonBar ()->recordButton ()->hide ();
+    showplaylistbutton = m_config->readBoolEntry (strAddPlaylistButton, true);
     seektime = m_config->readNumEntry (strSeekTime, 10);
     dvddevice = m_config->readEntry (strDVDDevice, "/dev/dvd");
     vcddevice = m_config->readEntry (strVCDDevice, "/dev/cdrom");
     videodriver = m_config->readNumEntry (strVoDriver, 0);
     audiodriver = m_config->readNumEntry (strAoDriver, 0);
-    urlbackend = m_config->readEntry(strUrlBackend, "mplayer");
     allowhref = m_config->readBoolEntry(strAllowHref, false);
 
     // recording
@@ -233,13 +350,21 @@ KDE_NO_EXPORT void KMPlayerSettings::readConfig () {
     pp_med_int = m_config->readBoolEntry (strPP_Med_Int, false);
     pp_ffmpeg_int = m_config->readBoolEntry (strPP_FFmpeg_Int, false);
 
-    std::for_each (pagelist.begin (), pagelist.end (),
-        std::bind2nd (std::mem_fun (&KMPlayerPreferencesPage::read), m_config));
+    for (PreferencesPage * p = pagelist; p; p = p->next)
+        p->read (m_config);
+    emit configChanged ();
 }
 
-KDE_NO_EXPORT bool KMPlayerSettings::createDialog () {
+KDE_NO_EXPORT bool Settings::createDialog () {
     if (configdialog) return false;
-    configdialog = new KMPlayerPreferences (m_player, this);
+    configdialog = new Preferences (m_player, this);
+    int id = 0;
+    const PartBase::ProcessMap::const_iterator e = m_player->players ().end ();
+    for (PartBase::ProcessMap::const_iterator i = m_player->players ().begin(); i != e; ++i) {
+        Process * p = i.data ();
+        if (p->supports ("urlsource"))
+            configdialog->m_SourcePageURL->backend->insertItem (p->menuName ().remove (QChar ('&')), id++);
+    }
     connect (configdialog, SIGNAL (okClicked ()),
             this, SLOT (okPressed ()));
     connect (configdialog, SIGNAL (applyClicked ()),
@@ -250,45 +375,76 @@ KDE_NO_EXPORT bool KMPlayerSettings::createDialog () {
     return true;
 }
 
-void KMPlayerSettings::addPage (KMPlayerPreferencesPage * page) {
-    if (std::find (pagelist.begin (), pagelist.end (), page) != pagelist.end ())
-        return;
+void Settings::addPage (PreferencesPage * page) {
+    for (PreferencesPage * p = pagelist; p; p = p->next)
+        if (p == page)
+            return;
     page->read (m_config);
     if (configdialog) {
         configdialog->addPrefPage (page);
         page->sync (false);
     }
-    pagelist.push_front (page);
+    page->next = pagelist;
+    pagelist = page;
 }
 
-void KMPlayerSettings::removePage (KMPlayerPreferencesPage * page) {
+void Settings::removePage (PreferencesPage * page) {
     if (configdialog)
         configdialog->removePrefPage (page);
-    pagelist.remove (page);
+    PreferencesPage * prev = 0L;
+    for (PreferencesPage * p = pagelist; p; prev = p, p = p->next)
+        if (p == page) {
+            if (prev)
+                prev->next = p->next;
+            else
+                pagelist = p->next;
+            break;
+        }
 }
     
-void KMPlayerSettings::show (const char * pagename) {
+void Settings::show (const char * pagename) {
     bool created = createDialog ();
     configdialog->m_GeneralPageGeneral->keepSizeRatio->setChecked (sizeratio);
-    configdialog->m_GeneralPageGeneral->showConsoleOutput->setChecked (showconsole);
+    configdialog->m_GeneralPageGeneral->autoResize->setChecked (autoresize);
+    configdialog->m_GeneralPageGeneral->sizesChoice->setButton (remembersize ? 0 : 1);
+    configdialog->m_GeneralPageGeneral->dockSysTray->setChecked (docksystray);
     configdialog->m_GeneralPageGeneral->loop->setChecked (loop);
     configdialog->m_GeneralPageGeneral->framedrop->setChecked (framedrop);
+    configdialog->m_GeneralPageGeneral->adjustvolume->setChecked (autoadjustvolume);
+    configdialog->m_GeneralPageGeneral->adjustcolors->setChecked (autoadjustcolors);
     //configdialog->m_GeneralPageGeneral->autoHideSlider->setChecked (autohideslider);
-    //configdialog->addConfigButton->setChecked (showcnfbutton);	//not
+    configdialog->m_GeneralPageGeneral->showConfigButton->setChecked (showcnfbutton);
+    configdialog->m_GeneralPageGeneral->showPlaylistButton->setChecked (showplaylistbutton);
     configdialog->m_GeneralPageGeneral->showRecordButton->setChecked (showrecordbutton);
     configdialog->m_GeneralPageGeneral->showBroadcastButton->setChecked (showbroadcastbutton);
     configdialog->m_GeneralPageGeneral->seekTime->setValue(seektime);
+    for (int i = 0; i < int (ColorSetting::last_target); i++)
+        colors[i].newcolor = colors[i].color;
+    for (int i = 0; i < int (FontSetting::last_target); i++)
+        fonts[i].newfont = fonts[i].font;
     configdialog->m_SourcePageURL->urllist->clear ();
     configdialog->m_SourcePageURL->urllist->insertStringList (urllist);
-    configdialog->m_SourcePageURL->urllist->setCurrentText (m_player->process ()->source ()->url ().prettyURL ());
+    configdialog->m_SourcePageURL->urllist->setCurrentText (m_player->source ()->url ().prettyURL ());
     configdialog->m_SourcePageURL->sub_urllist->clear ();
     configdialog->m_SourcePageURL->sub_urllist->insertStringList (sub_urllist);
-    configdialog->m_SourcePageURL->sub_urllist->setCurrentText (m_player->process ()->source ()->subUrl ().prettyURL ());
+    configdialog->m_SourcePageURL->sub_urllist->setCurrentText (m_player->source ()->subUrl ().prettyURL ());
     configdialog->m_SourcePageURL->changed = false;
+    configdialog->m_SourcePageURL->prefBitRate->setText (QString::number (prefbitrate));
+    configdialog->m_SourcePageURL->maxBitRate->setText (QString::number (maxbitrate));
 
     configdialog->m_GeneralPageOutput->videoDriver->setCurrentItem (videodriver);
     configdialog->m_GeneralPageOutput->audioDriver->setCurrentItem (audiodriver);
-    configdialog->m_SourcePageURL->backend->setCurrentItem (configdialog->m_SourcePageURL->backend->findItem (urlbackend));
+    configdialog->m_SourcePageURL->backend->setCurrentItem (configdialog->m_SourcePageURL->backend->findItem (backends["urlsource"]));
+    int id = 0;
+    const PartBase::ProcessMap::const_iterator e = m_player->players ().end ();
+    for (PartBase::ProcessMap::const_iterator i = m_player->players ().begin(); i != e; ++i) {
+        Process * p = i.data ();
+        if (p->supports ("urlsource")) {
+            if (backends["urlsource"] == QString (p->name()))
+                configdialog->m_SourcePageURL->backend->setCurrentItem (id);
+            id++;
+        }
+    }
     configdialog->m_SourcePageURL->allowhref->setChecked (allowhref);
 
     // postproc
@@ -327,6 +483,7 @@ void KMPlayerSettings::show (const char * pagename) {
     configdialog->m_RecordPage->replay->setButton (int (replayoption));
     configdialog->m_RecordPage->recorder->setButton (int (recorder));
     configdialog->m_RecordPage->replayClicked (int (replayoption));
+    configdialog->m_RecordPage->recorderClicked (int (recorder));
     configdialog->m_RecordPage->replaytime->setText (QString::number (replaytime));
     configdialog->m_MEncoderPage->arguments->setText (mencoderarguments);
     configdialog->m_MEncoderPage->format->setButton (recordcopy ? 0 : 1);
@@ -334,8 +491,8 @@ void KMPlayerSettings::show (const char * pagename) {
     configdialog->m_FFMpegPage->arguments->setText (ffmpegarguments);
 
     //dynamic stuff
-    std::for_each (pagelist.begin (), pagelist.end (),
-        std::bind2nd (std::mem_fun (&KMPlayerPreferencesPage::sync), false));
+    for (PreferencesPage * p = pagelist; p; p = p->next)
+        p->sync (false);
     //\dynamic stuff
     if (pagename)
         configDialog ()->setPage (pagename);
@@ -344,27 +501,39 @@ void KMPlayerSettings::show (const char * pagename) {
     configdialog->show ();
 }
 
-void KMPlayerSettings::writeConfig () {
-    KMPlayerView *view = static_cast <KMPlayerView *> (m_player->view ());
-
+void Settings::writeConfig () {
     m_config->setGroup (strGeneralGroup);
     m_config->writeEntry (strURLList, urllist, ';');
     m_config->writeEntry (strSubURLList, sub_urllist, ';');
+    m_config->writeEntry (strPrefBitRate, prefbitrate);
+    m_config->writeEntry (strMaxBitRate, maxbitrate);
+    m_config->writeEntry (strVolume, volume);
     m_config->writeEntry (strContrast, contrast);
     m_config->writeEntry (strBrightness, brightness);
     m_config->writeEntry (strHue, hue);
     m_config->writeEntry (strSaturation, saturation);
+    const QMap<QString,QString>::iterator b_end = backends.end ();
+    for (QMap<QString,QString>::iterator i = backends.begin(); i != b_end; ++i)
+        m_config->writeEntry (i.key (), i.data ());
+    for (int i = 0; i < int (ColorSetting::last_target); i++)
+        m_config->writeEntry (colors[i].option, colors[i].color);
+    for (int i = 0; i < int (FontSetting::last_target); i++)
+        m_config->writeEntry (fonts[i].option, fonts[i].font);
     m_config->setGroup (strMPlayerGroup);
-    m_config->writeEntry (strKeepSizeRatio, view->keepSizeRatio ());
-    m_config->writeEntry (strShowConsole, view->showConsoleOutput());
+    m_config->writeEntry (strKeepSizeRatio, sizeratio);
+    m_config->writeEntry (strAutoResize, autoresize);
+    m_config->writeEntry (strRememberSize, remembersize);
+    m_config->writeEntry (strDockSysTray, docksystray);
     m_config->writeEntry (strLoop, loop);
     m_config->writeEntry (strFrameDrop, framedrop);
+    m_config->writeEntry (strAdjustVolume, autoadjustvolume);
+    m_config->writeEntry (strAdjustColors, autoadjustcolors);
     m_config->writeEntry (strSeekTime, seektime);
     m_config->writeEntry (strVoDriver, videodriver);
     m_config->writeEntry (strAoDriver, audiodriver);
-    m_config->writeEntry (strUrlBackend, urlbackend);
     m_config->writeEntry (strAllowHref, allowhref);
     m_config->writeEntry (strAddConfigButton, showcnfbutton);
+    m_config->writeEntry (strAddPlaylistButton, showplaylistbutton);
     m_config->writeEntry (strAddRecordButton, showrecordbutton);
     m_config->writeEntry (strAddBroadcastButton, showbroadcastbutton);
 
@@ -414,17 +583,15 @@ void KMPlayerSettings::writeConfig () {
     m_config->writeEntry (strFFMpegArgs, ffmpegarguments);
 
     //dynamic stuff
-    std::for_each (pagelist.begin (), pagelist.end (),
-        std::bind2nd (std::mem_fun (&KMPlayerPreferencesPage::write), m_config));
+    for (PreferencesPage * p = pagelist; p; p = p->next)
+        p->write (m_config);
     //\dynamic stuff
     m_config->sync ();
 }
 
-KDE_NO_EXPORT void KMPlayerSettings::okPressed () {
-    KMPlayerView *view = static_cast <KMPlayerView *> (m_player->view ());
-    if (!view)
-        return;
+KDE_NO_EXPORT void Settings::okPressed () {
     bool urlchanged = configdialog->m_SourcePageURL->changed;
+    bool playerchanged = false;
     if (urlchanged) {
         if (configdialog->m_SourcePageURL->url->url ().isEmpty ())
             urlchanged = false;
@@ -477,31 +644,38 @@ KDE_NO_EXPORT void KMPlayerSettings::okPressed () {
     for (int i = 0; i < configdialog->m_SourcePageURL->sub_urllist->count () && i < 20; ++i)
         if (!configdialog->m_SourcePageURL->sub_urllist->text (i).isEmpty ())
             sub_urllist.push_back (configdialog->m_SourcePageURL->sub_urllist->text (i));
+    prefbitrate = configdialog->m_SourcePageURL->prefBitRate->text ().toInt ();
+    maxbitrate = configdialog->m_SourcePageURL->maxBitRate->text ().toInt ();
     sizeratio = configdialog->m_GeneralPageGeneral->keepSizeRatio->isChecked ();
-    m_player->keepMovieAspect (sizeratio);
-    showconsole = configdialog->m_GeneralPageGeneral->showConsoleOutput->isChecked ();
-    view->setShowConsoleOutput (showconsole);
+    autoresize = configdialog->m_GeneralPageGeneral->autoResize->isChecked ();
+    remembersize=!configdialog->m_GeneralPageGeneral->sizesChoice->selectedId();
+    docksystray = configdialog->m_GeneralPageGeneral->dockSysTray->isChecked ();
     loop = configdialog->m_GeneralPageGeneral->loop->isChecked ();
     framedrop = configdialog->m_GeneralPageGeneral->framedrop->isChecked ();
-    //showcnfbutton = configdialog->m_GeneralPageGeneral->addConfigButton->isChecked ();
-    showcnfbutton = true;
-    if (showcnfbutton)
-	view->buttonBar ()->configButton ()->show ();
-    else
-        view->buttonBar ()->configButton ()->hide ();
+    autoadjustvolume = configdialog->m_GeneralPageGeneral->adjustvolume->isChecked ();
+    autoadjustcolors = configdialog->m_GeneralPageGeneral->adjustcolors->isChecked ();
+    showcnfbutton = configdialog->m_GeneralPageGeneral->showConfigButton->isChecked ();
+    showplaylistbutton = configdialog->m_GeneralPageGeneral->showPlaylistButton->isChecked ();
     showrecordbutton = configdialog->m_GeneralPageGeneral->showRecordButton->isChecked ();
-    if (showrecordbutton)
-	view->buttonBar ()->recordButton ()->show ();
-    else
-        view->buttonBar ()->recordButton ()->hide ();
     showbroadcastbutton = configdialog->m_GeneralPageGeneral->showBroadcastButton->isChecked ();
-    if (!showbroadcastbutton)
-        view->buttonBar ()->broadcastButton ()->hide ();
     seektime = configdialog->m_GeneralPageGeneral->seekTime->value();
 
     videodriver = configdialog->m_GeneralPageOutput->videoDriver->currentItem();
     audiodriver = configdialog->m_GeneralPageOutput->audioDriver->currentItem();
-    urlbackend = configdialog->m_SourcePageURL->backend->currentText ();
+    if (!strcmp (m_player->source()->name (), "urlsource")) {
+        int backend = configdialog->m_SourcePageURL->backend->currentItem ();
+        const PartBase::ProcessMap::const_iterator e = m_player->players ().end();
+        for (PartBase::ProcessMap::const_iterator i = m_player->players ().begin(); backend >=0 && i != e; ++i) {
+            Process * proc = i.data ();
+            if (proc->supports ("urlsource") && backend-- == 0) {
+                backends["urlsource"] = proc->name ();
+                if (proc != m_player->process ()) {
+                    m_player->setProcess (proc->name ());
+                    playerchanged = true;
+                }
+            }
+        }
+    }
     allowhref = configdialog->m_SourcePageURL->allowhref->isChecked ();
     //postproc
     postprocessing = configdialog->m_OPPagePostproc->postProcessing->isChecked();
@@ -552,22 +726,22 @@ KDE_NO_EXPORT void KMPlayerSettings::okPressed () {
 #endif
 
     //dynamic stuff
-    std::for_each (pagelist.begin (), pagelist.end (),
-        std::bind2nd (std::mem_fun (&KMPlayerPreferencesPage::sync), true));
+    for (PreferencesPage * p = pagelist; p; p = p->next)
+        p->sync (true);
     //\dynamic stuff
 
     writeConfig ();
     emit configChanged ();
 
-    if (urlchanged) {
-        m_player->urlSource ()->setSubURL
+    if (urlchanged || playerchanged) {
+        m_player->sources () ["urlsource"]->setSubURL
             (KURL(configdialog->m_SourcePageURL->sub_url->url()));
         m_player->openURL (KURL::fromPathOrURL (configdialog->m_SourcePageURL->url->url ()));
-        m_player->process ()->source ()->setSubURL (KURL::fromPathOrURL (configdialog->m_SourcePageURL->sub_url->url ()));
+        m_player->source ()->setSubURL (KURL::fromPathOrURL (configdialog->m_SourcePageURL->sub_url->url ()));
     }
 }
 
-KDE_NO_EXPORT void KMPlayerSettings::getHelp () {
+KDE_NO_EXPORT void Settings::getHelp () {
     KApplication::kApplication()->invokeBrowser ("man:/mplayer");
 }
 
