@@ -13,14 +13,15 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
-#include <config.h>
+#include <libgen.h>
 #include <dcopclient.h>
 #include <qcstring.h>
 #include <qtimer.h>
@@ -58,20 +59,9 @@ typedef struct {
     uint32_t  status;
 } MWMHints;
 
-class KXinePlayerPrivate {
-public:
-    KXinePlayerPrivate()
-       : vo_driver ((char *) "auto"),
-         ao_driver ((char *) "auto"),
-         brightness (0), contrast (0), hue (0), saturation (0), volume (0) {
-    }
-    char * vo_driver;
-    char * ao_driver;
-    int brightness, contrast, hue, saturation, volume;
-};
 
 static KXinePlayer * xineapp;
-static KMPlayerCallback_stub * callback;
+static KMPlayer::Callback_stub * callback;
 static QMutex mutex (true);
 
 static xine_t              *xine;
@@ -83,8 +73,6 @@ static xine_post_t         *post_plugin;
 static xine_event_queue_t  *event_queue;
 static xine_cfg_entry_t     audio_vis_cfg_entry;
 static x11_visual_t         vis;
-static char                *dvd_device;
-static char                *vcd_device;
 static char                 configfile[2048];
 
 static Display             *display;
@@ -98,17 +86,26 @@ static int                  screen;
 static int                  completion_event;
 static int                  xpos, ypos, width, height;
 static int                  movie_width, movie_height, movie_length, movie_pos;
+static int                  movie_brightness = 32767;
+static int                  movie_contrast = 32767;
+static int                  movie_hue = 32767;
+static int                  movie_saturation = 32767;
+static int                  movie_volume = 32767;
 static double               pixel_aspect;
 
 static int                  running = 0;
-static int                  firstframe = 0;
+static volatile int         firstframe = 0;
 static const int            event_finished = QEvent::User;
 static const int            event_progress = QEvent::User + 2;
 static const int            event_url = QEvent::User + 3;
 static const int            event_size = QEvent::User + 4;
 static const int            event_title = QEvent::User + 5;
+static const int            event_video = QEvent::User + 6;
 static QString mrl;
 static QString sub_mrl;
+static QString rec_mrl;
+static QString alang, slang;
+static QStringList alanglist, slanglist;
 
 static QString elmentry ("entry");
 static QString elmitem ("item");
@@ -139,6 +136,7 @@ static void frame_output_cb(void * /*data*/, int /*video_width*/, int /*video_he
         int *dest_width, int *dest_height, 
         double *dest_pixel_aspect, int *win_x, int *win_y) {
     if (running && firstframe) {
+        firstframe = 0;
         int pos;
         fprintf(stderr, "first frame\n");
         mutex.lock ();
@@ -146,8 +144,8 @@ static void frame_output_cb(void * /*data*/, int /*video_width*/, int /*video_he
         movie_width = xine_get_stream_info(stream, XINE_STREAM_INFO_VIDEO_WIDTH);
         movie_height = xine_get_stream_info(stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
         mutex.unlock ();
-        QApplication::postEvent (xineapp, new XineSizeEvent (movie_length, movie_width, movie_height, true));
-        firstframe = 0;
+        QApplication::postEvent (xineapp, new XineMovieParamEvent (movie_length, movie_width, movie_height, alanglist, slanglist, true));
+        
     }
 
     *dest_x            = 0;
@@ -203,25 +201,55 @@ static void event_listener(void * /*user_data*/, const xine_event_t *event) {
             }
             break;
         case XINE_EVENT_UI_CHANNELS_CHANGED: {
-            fprintf (stderr, "Channel changed event\n");
+            fprintf (stderr, "Channel changed event %d\n", firstframe);
             mutex.lock ();
             int w = xine_get_stream_info(stream, XINE_STREAM_INFO_VIDEO_WIDTH);
             int h = xine_get_stream_info(stream, XINE_STREAM_INFO_VIDEO_HEIGHT);
-            int pos, l;
+            int pos, l, nr;
             xine_get_pos_length (stream, 0, &pos, &l);
+            char * langstr = new char [66];
+            alanglist.clear ();
+            slanglist.clear ();
+
+            nr =xine_get_stream_info(stream,XINE_STREAM_INFO_MAX_AUDIO_CHANNEL);
+            // if nrch > 25) nrch = 25
+            for (int i = 0; i < nr; ++i) {
+                if (!xine_get_audio_lang (stream, i, langstr))
+                    continue;
+                QString ls = QString::fromLocal8Bit (langstr).stripWhiteSpace();
+                if (ls.isEmpty ())
+                    continue;
+                if (!slang.isEmpty () && alang == ls)
+                    xine_set_param(stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, i);
+                alanglist.push_back (ls);
+                fprintf (stderr, "alang %s\n", langstr);
+            }
+            nr = xine_get_stream_info(stream, XINE_STREAM_INFO_MAX_SPU_CHANNEL);
+            // if nrch > 25) nrch = 25
+            for (int i = 0; i < nr; ++i) {
+                if (!xine_get_spu_lang (stream, i, langstr))
+                    continue;
+                QString ls = QString::fromLocal8Bit (langstr).stripWhiteSpace();
+                if (ls.isEmpty ())
+                    continue;
+                if (!slang.isEmpty () && slang == ls)
+                    xine_set_param (stream, XINE_PARAM_SPU_CHANNEL, i);
+                slanglist.push_back (ls);
+                fprintf (stderr, "slang %s\n", langstr);
+            }
+            delete langstr;
             mutex.unlock ();
-            if (w > 0 && h > 0 &&
-                (l != movie_length || w != movie_width || h != movie_height)) {
-                movie_width = w;
-                movie_height = h;
-                movie_length = l;
-                QApplication::postEvent (xineapp, new XineSizeEvent (l, w, h));
-                if (window_created) {
-                    XLockDisplay (display);
-                    XResizeWindow (display, wid, movie_width, movie_height);
-                    XFlush (display);
-                    XUnlockDisplay (display);
-                }
+            movie_width = w;
+            movie_height = h;
+            movie_length = l;
+            QApplication::postEvent (xineapp, new XineMovieParamEvent (l, w, h, alanglist, slanglist, firstframe));
+            if (running && firstframe)
+                firstframe = 0;
+            if (window_created && w > 0 && h > 0) {
+                XLockDisplay (display);
+                XResizeWindow (display, wid, movie_width, movie_height);
+                XFlush (display);
+                XUnlockDisplay (display);
             }
             break;
         }
@@ -235,57 +263,70 @@ static void event_listener(void * /*user_data*/, const xine_event_t *event) {
 
 } // extern "C"
 
-KMPlayerBackend::KMPlayerBackend ()
-    : DCOPObject (QCString ("KMPlayerBackend")) {
+using namespace KMPlayer;
+
+Backend::Backend ()
+    : DCOPObject (QCString ("Backend")) {
 }
 
-KMPlayerBackend::~KMPlayerBackend () {}
+Backend::~Backend () {}
 
-void KMPlayerBackend::setURL (QString url) {
+void Backend::setURL (QString url) {
     mrl = url;
 }
 
-void KMPlayerBackend::setSubTitleURL (QString url) {
+void Backend::setSubTitleURL (QString url) {
     sub_mrl = url;
 }
 
-void KMPlayerBackend::play () {
+void Backend::play () {
     xineapp->play ();
 }
 
-void KMPlayerBackend::stop () {
+void Backend::stop () {
     QTimer::singleShot (0, xineapp, SLOT (stop ()));
 }
 
-void KMPlayerBackend::pause () {
+void Backend::pause () {
     xineapp->pause ();
 }
 
-void KMPlayerBackend::seek (int pos, bool /*absolute*/) {
+void Backend::seek (int pos, bool /*absolute*/) {
     xineapp->seek (pos);
 }
 
-void KMPlayerBackend::hue (int h, bool) {
-    xineapp->hue (h);
+void Backend::hue (int h, bool) {
+    xineapp->hue (65535 * (h + 100) / 200);
 }
 
-void KMPlayerBackend::saturation (int s, bool) {
-    xineapp->saturation (s);
+void Backend::saturation (int s, bool) {
+    xineapp->saturation (65535 * (s + 100) / 200);
 }
 
-void KMPlayerBackend::contrast (int c, bool) {
-    xineapp->contrast (c);
+void Backend::contrast (int c, bool) {
+    xineapp->contrast (65535 * (c + 100) / 200);
 }
 
-void KMPlayerBackend::brightness (int b, bool) {
-    xineapp->brightness (b);
+void Backend::brightness (int b, bool) {
+    xineapp->brightness (65535 * (b + 100) / 200);
 }
 
-void KMPlayerBackend::volume (int v, bool) {
+void Backend::volume (int v, bool) {
     xineapp->volume (v);
 }
 
-void KMPlayerBackend::quit () {
+void Backend::frequency (int) {
+}
+
+void Backend::setAudioLang (int id, QString al) {
+    xineapp->setAudioLang (id, al);
+}
+
+void Backend::setSubtitle (int id, QString sl) {
+    xineapp->setSubtitle (id, sl);
+}
+
+void Backend::quit () {
     delete callback;
     callback = 0L;
     if (running)
@@ -312,7 +353,7 @@ bool updateConfigEntry (const QString & name, const QString & value) {
     return changed;
 }
 
-void KMPlayerBackend::setConfig (QByteArray data) {
+void Backend::setConfig (QByteArray data) {
     QString err;
     int line, column;
     QDomDocument dom;
@@ -333,65 +374,31 @@ void KMPlayerBackend::setConfig (QByteArray data) {
         callback->errorMessage (0, err);
 }
 
+bool Backend::isPlaying () {
+    mutex.lock ();
+    bool b = running &&
+        (xine_get_status (stream) == XINE_STATUS_PLAY) &&
+        (xine_get_param (stream, XINE_PARAM_SPEED) != XINE_SPEED_PAUSE);
+    mutex.unlock ();
+    return b;
+}
+
 KXinePlayer::KXinePlayer (int _argc, char ** _argv)
   : QApplication (_argc, _argv, false) {
-    d = new KXinePlayerPrivate;
-    window_created = true;
-    for(int i = 1; i < argc (); i++) {
-        if (!strcmp (argv ()[i], "-vo")) {
-            d->vo_driver = argv ()[++i];
-        } else if (!strcmp (argv ()[i], "-ao")) {
-            d->ao_driver = argv ()[++i];
-        } else if (!strcmp (argv ()[i], "-dvd-device")) {
-            dvd_device = argv ()[++i];
-        } else if (!strcmp (argv ()[i], "-vcd-device")) {
-            vcd_device = argv ()[++i];
-        } else if (!strcmp (argv ()[i], "-wid") || !strcmp (argv ()[i], "-window-id")) {
-            wid = atol (argv ()[++i]);
-            window_created = false;
-        } else if (!strcmp (argv ()[i], "-root")) {
-            wid =  XDefaultRootWindow (display);
-            window_created = false;
-        } else if (!strcmp (argv ()[i], "-window")) {
-            ;
-        } else if (!strcmp (argv ()[i], "-sub")) {
-            sub_mrl = QString (argv ()[++i]);
-        } else if (!strcmp (argv ()[i], "-v")) {
-            xine_verbose = true;
-        } else if (!strcmp (argv ()[i], "-vv")) {
-            xine_verbose = xine_vverbose = true;
-        } else if (!strcmp (argv ()[i], "-c")) {
-            wants_config = true;
-        } else if (!strcmp (argv ()[i], "-f") && i < argc () - 1) {
-            strncpy (configfile, argv ()[++i], sizeof (configfile));
-            configfile[sizeof (configfile) - 1] = 0;
-        } else if (!strcmp (argv ()[i], "-cb")) {
-            QString str = argv ()[++i];
-            int pos = str.find ('/');
-            if (pos > -1) {
-                fprintf (stderr, "callback is %s %s\n", str.left (pos).ascii (), str.mid (pos + 1).ascii ());
-                callback = new KMPlayerCallback_stub 
-                    (str.left (pos).ascii (), str.mid (pos + 1).ascii ());
-            }
-        } else 
-            mrl = QString::fromLocal8Bit (argv ()[i]);
-    }
+}
+
+void KXinePlayer::init () {
     xpos    = 0;
     ypos    = 0;
     width   = 320;
     height  = 200;
 
+    XLockDisplay(display);
     if (window_created)
         wid = XCreateSimpleWindow(display, XDefaultRootWindow(display),
                 xpos, ypos, width, height, 1, 0, 0);
-    if (!callback)
-        QTimer::singleShot (10, this, SLOT (play ()));
     XSelectInput (display, wid,
                   (PointerMotionMask | ExposureMask | KeyPressMask | ButtonPressMask | StructureNotifyMask)); // | SubstructureNotifyMask));
-}
-
-void KXinePlayer::init () {
-    XLockDisplay(display);
     XWindowAttributes attr;
     XGetWindowAttributes(display, wid, &attr);
     width = attr.width;
@@ -419,9 +426,6 @@ void KXinePlayer::init () {
     //if(fabs(pixel_aspect - 1.0) < 0.01)
         pixel_aspect = 1.0;
 
-    vo_port = xine_open_video_driver(xine, d->vo_driver, XINE_VISUAL_TYPE_X11, (void *) &vis);
-
-    ao_port = xine_open_audio_driver (xine, d->ao_driver, NULL);
     const char *const * pp = xine_list_post_plugins_typed (xine, XINE_POST_TYPE_AUDIO_VISUALIZATION);
     int i;
     for (i = 0; pp[i]; i++);
@@ -431,15 +435,11 @@ void KXinePlayer::init () {
         options[i+1] = pp[i];
     options[i+1] = 0L;
     xine_config_register_enum (xine, "audio.visualization", 0, (char ** ) options, 0L, 0L, 0, xine_config_cb, 0L);
+    if (!callback)
+        QTimer::singleShot (10, this, SLOT (play ()));
 }
 
 KXinePlayer::~KXinePlayer () {
-    if (xine) {
-        mutex.lock ();
-        xine_close_audio_driver (xine, ao_port);  
-        xine_close_video_driver (xine, vo_port);  
-        mutex.unlock ();
-    }
     if (window_created) {
         XLockDisplay (display);
         fprintf (stderr, "unmap %lu\n", wid);
@@ -486,6 +486,8 @@ void getConfigEntries (QByteArray & buf) {
                 case XINE_CONFIG_TYPE_BOOL:
                     elm.setAttribute (atttype, valbool);
                     break;
+                default:
+                    fprintf (stderr, "unhandled config type: %d\n", entry.type);
             }
         }
         if (entry.help)
@@ -512,27 +514,58 @@ void KXinePlayer::play () {
     movie_width = 0;
     movie_height = 0;
 
+    if (mrl.startsWith ("cdda://"))
+        mrl = QString ("cdda:/") + mrl.mid (7);
     stream = xine_stream_new (xine, ao_port, vo_port);
     event_queue = xine_event_new_queue (stream);
     xine_event_create_listener_thread (event_queue, event_listener, NULL);
+    if (mrl == "cdda:/") {
+        int nr;
+        char ** mrls = xine_get_autoplay_mrls (xine, "CD", &nr);
+        running = 1;
+        for (int i = 0; i < nr; i++) {
+            QString m (mrls[i]);
+            QString title;
+            if (xine_open (stream, mrls[i])) {
+                const char * t = xine_get_meta_info (stream, XINE_META_INFO_TITLE);
+                if (t && t[0])
+                    title = QString::fromUtf8 (t);
+                xine_close (stream);
+            }
+            if (callback)
+                callback->subMrl (m, title);
+            else
+                printf ("track %s\n", m.utf8 ().data ());
+        }
+        mutex.unlock ();
+        finished ();
+        return;
+    }
 
     xine_gui_send_vo_data(stream, XINE_GUI_SEND_VIDEOWIN_VISIBLE, (void *) 1);
 
-    if (d->saturation)
-        xine_set_param( stream, XINE_PARAM_VO_SATURATION, d->saturation);
-    if (d->brightness)
-        xine_set_param (stream, XINE_PARAM_VO_BRIGHTNESS, d->brightness);
-    if (d->contrast)
-        xine_set_param (stream, XINE_PARAM_VO_CONTRAST, d->contrast);
-    if (d->hue)
-        xine_set_param (stream, XINE_PARAM_VO_HUE, d->hue);
     running = 1;
-    if (!xine_open (stream, (const char *) mrl.local8Bit ())) {
+    QString mrlsetup = mrl;
+    if (!rec_mrl.isEmpty ()) {
+        char * rm = strdup (rec_mrl.local8Bit ());
+        char *bn = basename (rm);
+        char *dn = dirname (rm);
+        if (bn)
+            updateConfigEntry (QString ("media.capture.save_dir"), QString::fromLocal8Bit (dn));
+        mrlsetup += QString ("#save:") + QString::fromLocal8Bit (bn);
+        free (rm);
+    }
+    if (!xine_open (stream, (const char *) mrlsetup.local8Bit ())) {
         fprintf(stderr, "Unable to open mrl '%s'\n", (const char *) mrl.local8Bit ());
         mutex.unlock ();
         finished ();
         return;
     }
+    xine_set_param (stream, XINE_PARAM_VO_SATURATION, movie_saturation);
+    xine_set_param (stream, XINE_PARAM_VO_BRIGHTNESS, movie_brightness);
+    xine_set_param (stream, XINE_PARAM_VO_CONTRAST, movie_contrast);
+    xine_set_param (stream, XINE_PARAM_VO_HUE, movie_hue);
+
     if (!sub_mrl.isEmpty ()) {
         fprintf(stderr, "Using subtitles from '%s'\n", (const char *) sub_mrl.local8Bit ());
         sub_stream = xine_stream_new (xine, NULL, vo_port);
@@ -544,15 +577,19 @@ void KXinePlayer::play () {
             xine_dispose (sub_stream);
             sub_stream = 0L;
         }
-    }
+    } 
     if (!xine_play (stream, 0, 0)) {
         fprintf(stderr, "Unable to play mrl '%s'\n", (const char *) mrl.local8Bit ());
         mutex.unlock ();
         finished ();
         return;
     }
-    audio_vis = !xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_VIDEO);
-    audio_vis &= xine_config_lookup_entry (xine, "audio.visualization", &audio_vis_cfg_entry);
+    audio_vis = false;
+    if (xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_VIDEO))
+        QApplication::postEvent(xineapp, new QEvent((QEvent::Type)event_video));
+    else
+        audio_vis = xine_config_lookup_entry
+            (xine, "audio.visualization", &audio_vis_cfg_entry);
     mutex.unlock ();
     if (audio_vis)
         xine_config_cb (0L, &audio_vis_cfg_entry);
@@ -578,8 +615,12 @@ void KXinePlayer::postFinished () {
 void KXinePlayer::pause () {
     if (!running) return;
     mutex.lock ();
-    if (xine_get_status (stream) == XINE_STATUS_PLAY)
-        xine_set_param( stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
+    if (xine_get_status (stream) == XINE_STATUS_PLAY) {
+        if (xine_get_param (stream, XINE_PARAM_SPEED) == XINE_SPEED_PAUSE)
+            xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_NORMAL);
+        else
+            xine_set_param (stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE);
+    }
     mutex.unlock ();
 }
 
@@ -587,6 +628,22 @@ void KXinePlayer::finished () {
     QTimer::singleShot (10, this, SLOT (stop ()));
 }
 
+void KXinePlayer::setAudioLang (int id, const QString & al) {
+    alang = al;
+    mutex.lock ();
+    if (xine_get_status (stream) == XINE_STATUS_PLAY)
+        xine_set_param(stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, id);
+    mutex.unlock ();
+}
+
+void KXinePlayer::setSubtitle (int id, const QString & sl) {
+    slang = sl;
+    mutex.lock ();
+    if (xine_get_status (stream) == XINE_STATUS_PLAY)
+        xine_set_param (stream, XINE_PARAM_SPU_CHANNEL, id);
+    mutex.unlock ();
+}
+
 void KXinePlayer::updatePosition () {
     if (!running || !callback) return;
     int pos;
@@ -601,7 +658,7 @@ void KXinePlayer::updatePosition () {
 }
 
 void KXinePlayer::saturation (int val) {
-    d->saturation = val;
+    movie_saturation = val;
     if (running) {
         mutex.lock ();
         xine_set_param (stream, XINE_PARAM_VO_SATURATION, val);
@@ -610,7 +667,7 @@ void KXinePlayer::saturation (int val) {
 }
 
 void KXinePlayer::hue (int val) {
-    d->saturation = val;
+    movie_hue = val;
     if (running) {
         mutex.lock ();
         xine_set_param (stream, XINE_PARAM_VO_HUE, val);
@@ -619,7 +676,7 @@ void KXinePlayer::hue (int val) {
 }
 
 void KXinePlayer::contrast (int val) {
-    d->saturation = val;
+    movie_contrast = val;
     if (running) {
         mutex.lock ();
         xine_set_param (stream, XINE_PARAM_VO_CONTRAST, val);
@@ -628,7 +685,7 @@ void KXinePlayer::contrast (int val) {
 }
 
 void KXinePlayer::brightness (int val) {
-    d->saturation = val;
+    movie_brightness = val;
     if (running) {
         mutex.lock ();
         xine_set_param (stream, XINE_PARAM_VO_BRIGHTNESS, val);
@@ -637,7 +694,7 @@ void KXinePlayer::brightness (int val) {
 }
 
 void KXinePlayer::volume (int val) {
-    d->saturation = val;
+    movie_volume = val;
     if (running) {
         mutex.lock ();
         xine_set_param( stream, XINE_PARAM_AUDIO_VOLUME, val);
@@ -671,13 +728,15 @@ bool KXinePlayer::event (QEvent * e) {
                 xine_dispose (sub_stream);
                 sub_stream = 0L;
             }
-            xine_event_dispose_queue (event_queue);
-            xine_dispose (stream);
-            stream = 0L;
-            XLockDisplay (display);
-            XClearWindow (display, wid);
-            XUnlockDisplay (display);
+            if (stream) {
+                xine_event_dispose_queue (event_queue);
+                xine_dispose (stream);
+                stream = 0L;
+            }
             mutex.unlock ();
+            //XLockDisplay (display);
+            //XClearWindow (display, wid);
+            //XUnlockDisplay (display);
             if (callback)
                 callback->finished ();
             else
@@ -685,9 +744,10 @@ bool KXinePlayer::event (QEvent * e) {
             break;
         }
         case event_size: {
-            XineSizeEvent * se = static_cast <XineSizeEvent *> (e);                
             if (callback) {
-                callback->movieParams (se->length/100, se->width, se->height, se->height ? 1.0*se->width/se->height : 1.0);
+                XineMovieParamEvent * se = static_cast <XineMovieParamEvent *> (e);
+                if (se->length < 0) se->length = 0;
+                callback->movieParams (se->length/100, se->width, se->height, se->height ? 1.0*se->width/se->height : 1.0, se->alang, se->slang);
                 if (se->first_frame) {
                     callback->playing ();
                     QTimer::singleShot (500, this, SLOT (updatePosition ()));
@@ -704,24 +764,33 @@ bool KXinePlayer::event (QEvent * e) {
         case event_url: {
             XineURLEvent * ue = static_cast <XineURLEvent *> (e);                
             if (callback)
-                callback->statusMessage ((int) KMPlayerCallback::stat_addurl, ue->url);
+                callback->subMrl (ue->url, QString ());
             break;
         }
         case event_title: {
             XineTitleEvent * ue = static_cast <XineTitleEvent *> (e);                
             if (callback)
-                callback->statusMessage ((int) KMPlayerCallback::stat_newtitle, ue->title);
+                callback->statusMessage ((int) KMPlayer::Callback::stat_newtitle, ue->title);
             break;
         }
+        case event_video:
+            if (callback)
+                callback->statusMessage ((int) KMPlayer::Callback::stat_hasvideo, QString ());
+            break;
         default:
             return false;
     }
     return true;
 }
 
-XineSizeEvent::XineSizeEvent (int l, int w, int h, bool ff)
+void KXinePlayer::saveState (QSessionManager & sm) {
+    if (callback)
+        sm.setRestartHint (QSessionManager::RestartNever);
+}
+
+XineMovieParamEvent::XineMovieParamEvent(int l, int w, int h, const QStringList & a, const QStringList & s, bool ff)
   : QEvent ((QEvent::Type) event_size),
-    length (l), width (w), height (h), first_frame (ff) 
+    length (l), width (w), height (h), alang (a), slang (s) , first_frame (ff)
 {}
 
 XineURLEvent::XineURLEvent (const QString & u)
@@ -743,6 +812,9 @@ XineProgressEvent::XineProgressEvent (const int p)
 class XEventThread : public QThread {
 protected:
     void run () {
+        Time prev_click_time = 0;
+        int prev_click_x = 0;
+        int prev_click_y = 0;
         while (true) {
             XEvent   xevent;
             XNextEvent(display, &xevent);
@@ -904,7 +976,20 @@ protected:
                         mutex.unlock ();
                     }
                     break;
-                case ButtonPress:
+                case ButtonPress: {
+                    XButtonEvent *bev = (XButtonEvent *) &xevent;
+                    int dx = prev_click_x - bev->x;
+                    int dy = prev_click_y - bev->y;
+                    if (bev->time - prev_click_time < 400 &&
+                            (dx * dx + dy * dy) < 25) {
+                        xineapp->lock ();
+                        if (callback)
+                            callback->toggleFullScreen ();
+                        xineapp->unlock ();
+                    }
+                    prev_click_time = bev->time;
+                    prev_click_x = bev->x;
+                    prev_click_y = bev->y;
                     if (stream) {
                         fprintf(stderr, "ButtonPress\n");
                         XButtonEvent *bev = (XButtonEvent *) &xevent;
@@ -925,6 +1010,7 @@ protected:
                         mutex.unlock ();
                     }
                     break;
+                }
                 case NoExpose:
                     //fprintf (stderr, "NoExpose %lu\n", xevent.xnoexpose.drawable);
                     break;
@@ -949,6 +1035,9 @@ protected:
 };
 
 int main(int argc, char **argv) {
+    const char *dvd_device = 0L;
+    const char *vcd_device = 0L;
+    const char *grab_device = 0L;
     if (!XInitThreads ()) {
         fprintf (stderr, "XInitThreads () failed\n");
         return 1;
@@ -958,21 +1047,80 @@ int main(int argc, char **argv) {
 
     snprintf(configfile, sizeof (configfile), "%s%s", xine_get_homedir(), "/.xine/config2");
     xineapp = new KXinePlayer (argc, argv);
+    window_created = true;
+    QString vo_driver ("auto");
+    QString ao_driver ("auto");
+    for (int i = 1; i < argc; i++) {
+        if (!strcmp (argv [i], "-vo") && ++i < argc) {
+            vo_driver = argv [i];
+        } else if (!strcmp (argv [i], "-ao") && ++i < argc) {
+            ao_driver = argv [i];
+        } else if (!strcmp (argv [i], "-dvd-device") && ++i < argc) {
+            dvd_device = argv [i];
+        } else if (!strcmp (argv [i], "-vcd-device") && ++i < argc) {
+            vcd_device = argv [i];
+        } else if (!strcmp (argv [i], "-vd") && ++i < argc) {
+            grab_device = argv [i];
+        } else if ((!strcmp (argv [i], "-wid") ||
+                    !strcmp (argv [i], "-window-id")) && ++i < argc) {
+            wid = atol (argv [i]);
+            window_created = false;
+        } else if (!strcmp (argv [i], "-root")) {
+            wid =  XDefaultRootWindow (display);
+            window_created = false;
+        } else if (!strcmp (argv [i], "-window")) {
+            ;
+        } else if (!strcmp (argv [i], "-sub") && ++i < argc) {
+            sub_mrl = QString (argv [i]);
+        } else if (!strcmp (argv [i], "-lang") && ++i < argc) {
+            slang = alang = QString (argv [i]);
+        } else if (!strcmp (argv [i], "-v")) {
+            xine_verbose = true;
+        } else if (!strcmp (argv [i], "-vv")) {
+            xine_verbose = xine_vverbose = true;
+        } else if (!strcmp (argv [i], "-c")) {
+            wants_config = true;
+        } else if (!strcmp (argv [i], "-f") && ++i < argc) {
+            strncpy (configfile, argv [i], sizeof (configfile));
+            configfile[sizeof (configfile) - 1] = 0;
+        } else if (!strcmp (argv [i], "-cb") && ++i < argc) {
+            QString str = argv [i];
+            int pos = str.find ('/');
+            if (pos > -1) {
+                fprintf (stderr, "callback is %s %s\n", str.left (pos).ascii (), str.mid (pos + 1).ascii ());
+                callback = new KMPlayer::Callback_stub 
+                    (str.left (pos).ascii (), str.mid (pos + 1).ascii ());
+            }
+        } else if (!strcmp (argv [i], "-rec") && i < argc - 1) {
+            rec_mrl = QString::fromLocal8Bit (argv [++i]);
+        } else {
+            if (mrl.startsWith ("-session")) {
+                delete xineapp;
+                return 1;
+            }
+            mrl = QString::fromLocal8Bit (argv [i]);
+        }
+    }
     bool config_changed = !QFile (configfile).exists ();
 
     if (!callback && mrl.isEmpty ()) {
-        fprintf (stderr, "usage: %s [-vo (xv|xshm)] [-ao (arts|esd|..)] [-f <xine config file>] [-dvd-device <device>] [-vcd-device <device>] [-wid <X11 Window>|-window-id <X11 Window>|-root] [-sub <subtitle url>] [(-v|-vv)] [-cb <DCOP callback name> [-c]] [<url>]\n", argv[0]);
+        fprintf (stderr, "usage: %s [-vo (xv|xshm)] [-ao (arts|esd|..)] "
+                "[-f <xine config file>] [-dvd-device <device>] "
+                "[-vcd-device <device>] [-vd <video device>] "
+                "[-wid <X11 Window>|-window-id <X11 Window>|-root] "
+                "[-sub <subtitle url>] [-lang <lang>] [(-v|-vv)] "
+                "[-cb <DCOP callback name> [-c]] [<url>]\n", argv[0]);
         delete xineapp;
         return 1;
     }
 
-    DCOPClient dcopclient;
-    dcopclient.registerAs ("kxineplayer");
-    KMPlayerBackend player;
-
     XEventThread * eventThread = new XEventThread;
     eventThread->start ();
 
+    DCOPClient dcopclient;
+    dcopclient.registerAs ("kxineplayer");
+    Backend player;
+
     xine = xine_new();
     if (xine_verbose)
         xine_engine_set_param (xine, XINE_ENGINE_PARAM_VERBOSITY, xine_vverbose ? XINE_VERBOSITY_DEBUG : XINE_VERBOSITY_LOG);
@@ -985,24 +1133,77 @@ int main(int argc, char **argv) {
         config_changed |= updateConfigEntry (QString ("input.dvd_device"), QString (dvd_device));
     if (vcd_device)
         config_changed |= updateConfigEntry (QString ("input.vcd_device"), QString (vcd_device));
+    if (grab_device)
+        config_changed |= updateConfigEntry (QString ("media.video4linux.video_device"), QString (grab_device));
 
     if (config_changed)
         xine_config_save (xine, configfile);
 
-    if (callback) {
-        QByteArray buf;
-        if (wants_config)
-            getConfigEntries (buf);
-        callback->started (buf);
+    QStringList vos = QStringList::split (',', vo_driver);
+    for (int i = 0; i < vos.size (); i++) {
+        if (vos[i] == "x11")
+            vos[i] = "xshm";
+        else if (vos[i] == "gl")
+            vos[i] = "opengl";
+        fprintf (stderr, "trying video driver %s ..\n", vos[i].ascii ());
+        vo_port = xine_open_video_driver(xine, vos[i].ascii (),
+                XINE_VISUAL_TYPE_X11, (void *) &vis);
+        if (vo_port)
+            break;
     }
+    if (!vo_port)
+        fprintf (stderr, "no video driver found\n");
+    QStringList aos = QStringList::split (',', ao_driver);
+    for (int i = 0; i < aos.size (); i++) {
+        fprintf (stderr, "trying audio driver %s ..\n", aos[i].ascii ());
+        ao_port = xine_open_audio_driver (xine, aos[i].ascii (), NULL);
+        if (ao_port)
+            break;
+    }
+    if (!ao_port)
+        fprintf (stderr, "audio driver initialisation failed\n");
+    stream = xine_stream_new (xine, ao_port, vo_port);
+
+    QByteArray buf;
+    if (wants_config) {
+        /* TODO? Opening the output drivers in front, will add more config
+                 settings. Unfortunately, that also adds a second in startup..
+        const char *const * aops = xine_list_audio_output_plugins (xine);
+        for (const char *const* aop = aops; *aop; aop++) {
+            xine_audio_port_t * ap = xine_open_audio_driver (xine, *aop, 0L);
+            xine_close_audio_driver (xine, ap);
+            fprintf (stderr, "audio output: %s\n", *aop);
+        }
+        const char *const * vops = xine_list_video_output_plugins (xine);
+        for (const char *const* vop = vops; *vop; vop++) {
+            xine_video_port_t * vp = xine_open_video_driver (xine, *vop, XINE_VISUAL_TYPE_NONE, 0L);
+            xine_close_video_driver (xine, vp);
+            fprintf (stderr, "vidio output: %s\n", *vop);
+        }*/
+        getConfigEntries (buf);
+    }
+    if (callback)
+        callback->started (dcopclient.appId (), buf);
+    else
+        ;//printf ("%s\n", QString (buf).ascii ());
     xineapp->exec ();
 
+    if (sub_stream)
+        xine_dispose (sub_stream);
+    if (stream) {
+        xine_event_dispose_queue (event_queue);
+        xine_dispose (stream);
+    }
+    if (ao_port)
+        xine_close_audio_driver (xine, ao_port);  
+    if (vo_port)
+        xine_close_video_driver (xine, vo_port);  
     XLockDisplay(display);
     XClientMessageEvent ev = {
         ClientMessage, 0, true, display, wid, 
         XInternAtom (display, "XINE", false), 8, {b: "quit_now"}
     };
-    XSendEvent (display, wid, FALSE, StructureNotifyMask, (XEvent *) & ev);
+    XSendEvent (display, wid, false, StructureNotifyMask, (XEvent *) & ev);
     XFlush (display);
     XUnlockDisplay(display);
     eventThread->wait (500);
